﻿<#
.SYNOPSIS
Configure SQL Server

.PARAMETER $ConfigJSON
Full filename & path to the JSON configuration file.

.EXAMPLE
Include example text here:
 .\SQL_Config.ps1 -ConfigJSON "C:\Install\Config.json"

#>

Param ([string]$ConfigJSON
)

$Disclaimer = @"
###############################################################################################
# Notes:                                                                                      #
# 1. Verify SQL Service Acct has access to Backups location.                                  #
# 2. Verify SQL Service Acct has access to Certificate backup location.                       #
# 3. Verify SSL Certificate has been imported to the Local Machine certificate store.         #
# 4. Verify Service Account at Secpol.msc Local Policies/User Rights Assignment/Generate      #
#    Security Audits                                                                          #
###############################################################################################
Press ENTER to continue
"@

$continue = Read-Host $Disclaimer;

#########################
# Load JSON Config File #
#########################

$Config = ConvertFrom-Json -InputObject (Get-Content -Path "$ConfigJSON" -Raw);

###############################
# Set PowerShell Scripts Path #
###############################

$ScriptPath = (($Config).PowerShell).ScriptsPath;

#########################
# Set Config Files Path #
#########################

$ConfigFilesPath = Split-Path (($Config).SQLMedia).ConfigFile;

#######################################
# Get Installed SQL PowerShell Module #
#######################################

If (Get-Module -ListAvailable -Name SQLServer) {
    Import-Module Sqlserver;
    }
ElseIf (Get-Module -ListAvailable -Name SQLPS) {
    Write-Output "Config script not compatible with SQLPS module. Please install SQLServer PowerShell module.";
    Exit;
    }
Else {
    Write-Output "SQLServer PowerShell module not installed.";
    Exit;
}

#####################################
# Format Server names and Instances #
#####################################

$ServerList = ($Config).Servers;

ForEach ($Location in $ServerList) {

$Server = ($Location).ServerName;
$InstanceName = ($Location).InstanceName;
If ($InstanceName -eq "MSSQLSERVER") {
    $Instance = $Server;
}
Else {
$Instance = "$Server\$InstanceName";
}

#######################
# Set SA account name #
#######################

$SA = ((($Config).Accounts).SA).Username -replace '\$Server', $Server;

################################
# Get SQL Service Account Name #
################################

$SQLServiceAcct = (Invoke-Sqlcmd -ServerInstance "$Instance" -Query "SELECT [service_account] FROM [sys].[dm_server_services] WHERE [servicename] LIKE 'SQL Server ($InstanceName)'").service_account;
$SQLServiceAcct = $SQLServiceAcct.Substring($SQLServiceAcct.LastIndexOf('\')+1);

##############################
# Get SQL Agent Account Name #
##############################

$SQLAgentAcct = (Invoke-Sqlcmd -ServerInstance "$Instance" -Query "SELECT [service_account] FROM [sys].[dm_server_services] WHERE [servicename] LIKE 'SQL Server Agent ($InstanceName)'").service_account;
$SQLAgentAcct = $SQLAgentAcct.Substring($SQLAgentAcct.LastIndexOf('\')+1);

#################################
# Configure default backup path #
#################################
write-output "Configure default backup path";
$BackupDrive = (($Config).BackupLocation).Drive;
$BackupFolderPath = (($Config).BackupLocation).Folder;
$BackupPath = "$BackupDrive$BackupFolderPath";
$query = @"
USE [master]
GO
EXEC xp_instance_regwrite N'HKEY_LOCAL_MACHINE', N'Software\Microsoft\MSSQLServer\MSSQLServer', N'BackupDirectory', REG_SZ, N'$BackupPath'
GO
"@;
Invoke-Sqlcmd -ServerInstance "$Instance" -Query $query -Verbose;

#######################
#  Set Server Memory  #
#######################
write-output "Setting min/max server memory";

If ((($Config).Memory).Enabled -eq $true) {
$MinMemory = (($Config).Memory).Min;
$MaxMemory = (($Config).Memory).Max;
$NetworkPacket = (($Config).Memory).NetworkPacket;

$query = @"
EXEC sys.sp_configure N'show advanced options', N'1'  RECONFIGURE WITH OVERRIDE
GO
EXEC sys.sp_configure N'priority boost', N'1'
GO
EXEC sys.sp_configure N'network packet size (B)', N'$NetworkPacket'
GO
EXEC sys.sp_configure N'min server memory (MB)', N'$MinMemory'
GO
EXEC sys.sp_configure N'max server memory (MB)', N'$MaxMemory'
GO
RECONFIGURE
GO
EXEC sys.sp_configure N'show advanced options', N'0'  RECONFIGURE WITH OVERRIDE
GO
"@
Invoke-Sqlcmd -ServerInstance "$Instance" -Query "$query" -Verbose;
}

#######################
#  Rename SA Account  #
#######################
If ($SA) {
    write-output "Updating SA login";
    $query = "ALTER LOGIN [sa] WITH NAME = [$SA]";
    Invoke-Sqlcmd -ServerInstance "$Instance" -Query "$query" -Verbose;
}

##############################
# Update SA Account Password #
##############################

If ($SApwd) {
    write-output "Updating SA password";
    $query = "ALTER LOGIN [$SA] WITH PASSWORD = N'$SApwd'";
    Invoke-Sqlcmd -ServerInstance "$Instance" -Query "$query" -Verbose;
}

#############################
#  Start SQL Agent Service  #
#############################
write-output "Start SQL Agent";
Invoke-Command -ComputerName $Server -ScriptBlock {
	$agentservice = get-service "SQL Server Agent (MSSQLSERVER)" -ErrorAction SilentlyContinue
		if ( $agentservice.status -eq "Stopped" )
		 {
				 "Restarting Service…"
				 start-service $agentservice.name
				 "Service Started"
		 }
};

#################################
#  Create SQL Server Operators  #
#################################
write-output "Create Operators";
$OperatorList = (($Config).Accounts).Operators;

ForEach ($Account in $OperatorList) {
$Operator = $Account.AccountName;
$OperatorEmail = $Account.EmailAddress;

$SQLOperator = @"
EXEC msdb.dbo.sp_add_operator @name=N'$Operator',
    @enabled=1,
	@weekday_pager_start_time=90000,
	@weekday_pager_end_time=180000,
	@saturday_pager_start_time=90000,
	@saturday_pager_end_time=180000,
	@sunday_pager_start_time=90000,
	@sunday_pager_end_time=180000,
	@pager_days=0,
	@email_address=N'$OperatorEmail',
	@category_name=N'[Uncategorized]'
"@;
Invoke-Sqlcmd -ServerInstance "$Instance" -Query "$SQLOperator" -Verbose;
}

##########################
#  Enable Database Mail  #
##########################

If ((($Config).DBMail).Enable -eq $true) {
write-output "Enable DBMail";
$AccountList = (($Config).DBMail).Accounts;

ForEach ($Account in $AccountList) {
$MailServer = $Account.Server;
$MailAccount = $Account.Name;
$MailDisplay = $Account.Display;
$MailAddress = $Account.Address;

$query = @"
sp_configure 'show advanced options',1
go
reconfigure with override
go
sp_configure 'Database Mail XPs',1
 go
reconfigure

 -- Create a Database Mail accounts
EXECUTE msdb.dbo.sysmail_add_account_sp
    @account_name = '$MailAccount',
    @description = 'Mail account for server administration.',
    @email_address = '$MailAddress',
    @display_name = '$MailDisplay',
    @mailserver_name = '$MailServer' ;

	-- Create a Database Mail profile
EXECUTE msdb.dbo.sysmail_add_profile_sp
    @profile_name = '$MailAccount',
    @description = 'Profile used for administrative mail.' ;

-- Add the accounts to the profile
EXECUTE msdb.dbo.sysmail_add_profileaccount_sp
    @profile_name = '$MailAccount',
    @account_name = '$MailAccount',
    @sequence_number =1 ;

-- Grant access to the profile to all users in the msdb database
EXECUTE msdb.dbo.sysmail_add_principalprofile_sp
    @profile_name = '$MailAccount',
    @principal_name = 'public',
    @is_default = 1 ;
"@;
Invoke-Sqlcmd -ServerInstance "$Instance" -Query "$query" -Verbose;
}
}

#########################
#  Configure SQL Agent  #
#########################
write-output "Configure SQL Agent";
$query = @"
EXEC master.dbo.sp_MSsetalertinfo @failsafeoperator=N'$Operator', @notificationmethod=1;
EXEC msdb.dbo.sp_set_sqlagent_properties @email_save_in_sent_folder=1, @databasemail_profile=N'$MailAccount', @use_databasemail=1;
"@;
Invoke-Sqlcmd -ServerInstance "$Instance" -Query "$query" -Verbose;

##############################
#  Create SQL Server Alerts  #
##############################
write-output "Create SQL Server Alerts";

# Update Operator Account Name In Alerts Script
[string]$AlertsFilename = "$ScriptPath\Config_Scripts\Alerts_Triggers.sql";
(Get-Content $AlertsFilename) | ForEach-Object {
    switch -regex($_) {
    # Update Operator Account Name
    ('@operator_name=N') {$_ -replace "@operator_name=N''", ("@operator_name=N'$Operator'")}
    # Keep unaltered lines
    Default {$_};
    }
} | Set-Content -Path "$ConfigFilesPath\Alerts_Triggers_$Server.sql" -Force;

# Create Alerts
Invoke-Sqlcmd -ServerInstance "$Instance" -InputFile "$ConfigFilesPath\Alerts_Triggers_$Server.sql" -Verbose;

# Create New Database Alert

# IF working with SQL 2017 or newer, you can "CREATE OR ALTER" and do away with the "IF EXISTS" block.
$query = @"
IF EXISTS (SELECT * FROM sys.server_triggers WHERE name = 'ddl_trig_new_database')
    DROP TRIGGER ddl_trig_new_database ON ALL SERVER;
GO

CREATE TRIGGER ddl_trig_new_database ON ALL SERVER FOR CREATE_DATABASE
    AS
    DECLARE @DBNAME varchar(200)
    DECLARE @BODY varchar(1000)
    SELECT @DBNAME = EVENTDATA().value('(/EVENT_INSTANCE/DatabaseName)[1]','nvarchar(max)')
    SET @BODY = 'Server = ' + @@SERVERNAME + CHAR(10) + CHAR(13) +
                'New Database Named = ' + @DBNAME + CHAR(10) + CHAR(13) +
                'Created By = ' + ORIGINAL_LOGIN() + CHAR(10) + CHAR(13) +
                'Date Created = ' + cast(getdate() as CHAR)
    EXEC msdb.dbo.sp_notify_operator
    	@name = N'$Operator',
               @subject = N'New Database Created',
               @body = @BODY
"@;
Invoke-Sqlcmd -ServerInstance "$Instance" -Query "$query" -Verbose;

#################
# Register SPNs #
#################

If ((($Config).SPN).RegisterSPN -eq $true) {
write-output "Register SPNs";

$P_query = @"
setspn -S MSSQLSvc/$Server.$env:userdnsdomain`:1433 $env:userdomain`\$SQLServiceAcct;
setspn -S MSSQLSvc/$Server.$env:userdnsdomain $env:userdomain`\$SQLServiceAcct;
"@;
Invoke-Expression $P_query;

# SSRS SPNs
If ((($Config).SSRS).Enable -eq $True) {
write-output "Register SSRS SPN"
$P_query = @"
setspn -S HTTP/$SQLServer.$env:userdnsdomain`:1433 $env:userdomain`\$SQLServiceAcct;
setspn -S HTTP/$SQLServer.$env:userdnsdomain $env:userdomain`\$SQLServiceAcct;
"@;
Invoke-Expression $P_query;
}
}

#####################
# Enable Delegation #
#####################
# Set service account as trusted for delegation in AD
If ((($Config).Delegation).TrustForDelegation -eq $True) {
    Import-Module ActiveDirectory;
    Set-ADAccountControl -Identity $Server -TrustedForDelegation $true;
}

##########################################################
# Compress Backups (default)                             #
# https://msdn.microsoft.com/en-us/library/bb933863.aspx #
##########################################################
write-output "Set default to Compress Backups";
$query = @"
EXEC sp_configure 'backup compression default', 1;
RECONFIGURE WITH OVERRIDE;
"@;
Invoke-Sqlcmd -ServerInstance "$Instance" -Query "$query" -Verbose;

###############################################
# Create SSISDB & Import Maintenance Projects #
###############################################
If ((($Config).SSIS).Enable -eq $True) {
write-output "Enable CLR";

$SSISDBPwd = (($Config).SSIS).DBPassword;

$query = @"
sp_configure 'show advanced options', 1;
GO
RECONFIGURE;
GO
sp_configure 'clr enabled', 1;
GO
RECONFIGURE;
GO
"@;
Invoke-Sqlcmd -ServerInstance "$Instance" -Query "$query" -Verbose;

write-output "Creating SSISDB Catalog";
$query = @"
# Load the IntegrationServices Assembly
[Reflection.Assembly]::LoadWithPartialName("Microsoft.SqlServer.Management.IntegrationServices")
# Store the IntegrationServices Assembly namespace
$ISNamespace = "Microsoft.SqlServer.Management.IntegrationServices"
# Create a connection to the server
$sqlConnectionString = "Data Source=" + $Instance + ";Initial Catalog=master;Integrated Security=SSPI;"
$sqlConnection = New-Object System.Data.SqlClient.SqlConnection $sqlConnectionString
# Create the Integration Services object
$integrationServices = New-Object $ISNamespace".IntegrationServices" $sqlConnection
# Create a new SSISDB Catalog
$catalog = New-Object $ISNamespace".Catalog" ($integrationServices, "SSISDB", "$SSISDBPwd")
$catalog.Create()
# Create Maintenance Packages folder
$folder = New-Object $ISNamespace".CatalogFolder" ($catalog, "Maintenance Packages", "SQL Server Maintenance Packages")
$folder.Create()
"@;
Invoke-Command -ComputerName $Server -ScriptBlock {"$query"};
}

###########################
#  Modify Model Database  #
###########################
$ModelFiles = (($Config).ModelDB);
write-output "Set Model DB Size";
ForEach ($LogicalFile in $ModelFiles) {
$FileName = ($LogicalFile).LogicalName;
$FileSize = ($LogicalFile).InitialSize;
$FileGrowth = ($LogicalFile).FileGrowth;

$query = @"
ALTER DATABASE [model] MODIFY FILE ( NAME = N'$FileName', SIZE = $FileSize , FILEGROWTH = $FileGrowth );
"@;
Invoke-Sqlcmd -ServerInstance "$Instance" -Query "$query" -Verbose;
}

#############################################################
# Create Security Audit for Password and Permission Changes #
#############################################################
write-output "Configure Auditing to Security Log";
$query = @"
USE [master]
GO

CREATE SERVER AUDIT [Security Log]
TO SECURITY_LOG
WITH
(	QUEUE_DELAY = 1000
	,ON_FAILURE = CONTINUE
)

CREATE SERVER AUDIT SPECIFICATION [Security]
FOR SERVER AUDIT [Security Log]
ADD (DATABASE_ROLE_MEMBER_CHANGE_GROUP),
ADD (SERVER_ROLE_MEMBER_CHANGE_GROUP),
ADD (DATABASE_PERMISSION_CHANGE_GROUP),
ADD (DATABASE_OBJECT_PERMISSION_CHANGE_GROUP),
ADD (SERVER_OBJECT_PERMISSION_CHANGE_GROUP),
ADD (SERVER_PERMISSION_CHANGE_GROUP),
ADD (LOGIN_CHANGE_PASSWORD_GROUP),
ADD (USER_CHANGE_PASSWORD_GROUP)
WITH (STATE = ON)
GO

ALTER SERVER AUDIT [Security Log] WITH (STATE = ON)
GO
"@
Invoke-Sqlcmd -ServerInstance "$Instance" -Query "$query" -Verbose;

#####################################
# Increase Max rows for Job History #
#####################################
write-output "Increase max rows for job history";
$query = "EXEC msdb.dbo.sp_set_sqlagent_properties @jobhistory_max_rows_per_job=300";
Invoke-Sqlcmd -ServerInstance "$Instance" -Query "$query" -Verbose;


########################################################
# Set Error Log Count to 40 & Create Job to cycle logs #
########################################################

write-output "Configure Error Logs";
$query = "EXEC xp_instance_regwrite N'HKEY_LOCAL_MACHINE', N'Software\Microsoft\MSSQLServer\MSSQLServer', N'NumErrorLogs', REG_DWORD, 40";
Invoke-Sqlcmd -ServerInstance "$Instance" -Query "$query" -Verbose;

# Create SQL Agent Job
Invoke-Command -ComputerName $Server -FilePath "$ScriptPath\Config_Scripts\Configure_SQL_Error_Logs.ps1" -ArgumentList "$Instance","$SA","$Operator";

##############################################
#  Enable Contained Database Authentication  #
##############################################

If ((($Config).ContainedDBAuth).Enable -eq $True) {
write-output "Enable Contained DB Authentication";
$query = @"
EXEC sys.sp_configure N'contained database authentication', N'1'
GO
RECONFIGURE WITH OVERRIDE
"@;
Invoke-Sqlcmd -ServerInstance "$Instance" -Query "$query" -Verbose;
}

####################################
# Create additional login accounts #
####################################

Write-Output "Create SQL Logins on $Instance.";
$LoginsList = (($Config).Accounts).Logins;
ForEach ($Account in $LoginsList) {
    If ($Account.AccountType -eq "WindowsGroup") {
        Add-SqlLogin -ServerInstance "$Instance" -LoginType $Account.AccountType -LoginName $Account.Username -GrantConnectSql -Enable;
        $svr = New-Object ('Microsoft.SqlServer.Management.Smo.Server') $Instance;
        $svrole = $svr.Roles | Where-Object {$_.Name -eq $Account.Role};
        $svrole.AddMember($Account.Username);
    }
    If ($Account.AccountType -eq "WindowsUser") {
        Add-SqlLogin -ServerInstance "$Instance" -LoginType $Account.AccountType -LoginName $Account.Username -GrantConnectSql -Enable;
        $svr = New-Object ('Microsoft.SqlServer.Management.Smo.Server') $Instance;
        $svrole = $svr.Roles | Where-Object {$_.Name -eq $Account.Role};
        $svrole.AddMember($Account.Username);
    }
        If ($Account.AccountType -eq "SqlLogin") {
        Add-SqlLogin -ServerInstance "$Instance" -LoginType $Account.AccountType -LoginName $Account.Username -EnforcePasswordPolicy -GrantConnectSql -Enable;
        $svr = New-Object ('Microsoft.SqlServer.Management.Smo.Server') $Instance;
        $svrole = $svr.Roles | Where-Object {$_.Name -eq $Account.Role};
        $svrole.AddMember($Account.Username);
    }
}

#######################
# Startup Trace Flags #
#######################

# TF 3226 - Suppress logging of successful backup events to the sql error log
write-output "Set Startup Trace Flags";

$AddTF = ";-T3226;-T4199;-T8152";

$SQLWMI = New-Object Microsoft.SqlServer.Management.Smo.Wmi.ManagedComputer $Server;
$WMISvc = $SQLWMI.Services | Where-Object {$_.name -eq "MSSQLSERVER"};
$WMISvc.StartupParameters = $WMISvc.StartupParameters + $AddTF;
$WMISvc.Alter();

##############################
# Import Management Policies #
##############################
write-output "Import Management Policies";

$PolicyList = Get-Content $ScriptPath\Policies\InstallOrder.txt;
$PolicyList | ForEach-Object { # List of policies to import
$currScript = $_;
Write-Output "$currScript;"
$Connectionstring = "server=$Instance;Trusted_Connection=true";
$Conn = New-Object Microsoft.SQlServer.Management.Sdk.Sfc.SqlStoreConnection($Connectionstring);
$PolicyStore = New-Object Microsoft.SqlServer.Management.DMF.PolicyStore($Conn);
$PolicyXmlPath = "$ScriptPath\Policies\$currScript";
$xmlReader = [System.Xml.XmlReader]::Create($PolicyXmlPath);
$PolicyStore.ImportPolicy($xmlReader, [Microsoft.SqlServer.Management.Dmf.ImportPolicyEnabledState]::Unchanged, $true, $true);
"***** Import Policy $currScript ***** $([Environment]::NewLine)" | Out-File "$ConfigFilesPath\Policy_Errors_$Instance.txt" -Append;
}

############
# AlwaysOn #
############
If ((($Config).AlwaysOn).Enable -eq $true) {
    # Enable AlwaysOn
    Enable-SQLAlwaysOn -ServerInstance $Instance -NoServiceRestart;
    # Set Endpoint variables
    If ($InstanceName -eq "MSSQLSERVER") {
        $EInstance = "$Server\Default";
        }
    Else {
        $EInstance = $Instance
        }
    $EndpointPath = (((($Config).AlwaysOn).Endpoint).Path) -replace '\$Instance', $EInstance;
    $EndpointName = ((($Config).AlwaysOn).Endpoint).Name;
    $EndpointPort = ((($Config).AlwaysOn).Endpoint).Port;
    $EndpointAlgorithm = ((($Config).AlwaysOn).Endpoint).EncryptionAlgorithm;
    $EndpointEncryption = ((($Config).AlwaysOn).Endpoint).Encryption;
    # Create database mirroring endpoint
    New-SqlHADREndpoint -Name "$EndpointName" -Path "$EndpointPath" -Port $EndpointPort -EncryptionAlgorithm $EndpointAlgorithm -Encryption $EndpointEncryption;
	# Grant Connect to service account
	Invoke-Sqlcmd -ServerInstance "$Instance" -Query "CREATE LOGIN [$SQLServiceAcct] FROM WINDOWS WITH DEFAULT_DATABASE=[master]";
	Invoke-Sqlcmd -ServerInstance "$Instance" -Query "GRANT CONNECT ON ENDPOINT::[HadrEndpoint] TO [$SQLServiceAcct]";
}

##################
# Create Mgmt DB #
##################
Write-Output "Create Management Database.";
$MgmtDB = (($Config).ManagementDB).DBName;
$query = "CREATE DATABASE [$MgmtDB];"
Invoke-Sqlcmd -ServerInstance "$Instance" -Query "$query";

#######################################
# Enable Encrypted Client Connections #
#######################################

Write-Output "Enable Encrypted Client Connections on $Server.";
Invoke-Command -ComputerName $Server -FilePath "$ScriptPath\Config_Scripts\EnableSSLConnection.ps1" -ArgumentList "$env:userdomain`\$SQLServiceAcct","$Server.$env:userdnsdomain";

##########################################################
# Enable Extended Proctection                            #
# https://msdn.microsoft.com/en-us/library/ff487261.aspx #
##########################################################

Write-Output "Enable Extended Protection on $Server.";
Invoke-Command -ComputerName $Server -ScriptBlock {
    $regkey = Get-ChildItem -Path "HKLM:\SOFTWARE\Microsoft\Microsoft SQL Server" -Recurse | Where-Object {$_.Name -like "*SuperSocketNetLib"};
    Set-ItemProperty -path $regKey.PSPath -Name ExtendedProtection -value "1";
    }

##########################################################
# Hide SQL Instance                                      #
# https://msdn.microsoft.com/en-us/library/ms179327.aspx #
##########################################################

Write-Output "Make SQL Server $Server Non-Browsable";
Invoke-Command -ComputerName $Server -ScriptBlock {
    $regkey = Get-ChildItem -Path "HKLM:\SOFTWARE\Microsoft\Microsoft SQL Server" -Recurse | Where-Object {$_.Name -like "*SuperSocketNetLib"};
    Set-ItemProperty -path $regKey.PSPath -Name HideInstance -value "1";
    }

#####################
# Backup Encryption #
#####################
If ((($Config).BackupEncryption).Enable -eq $True) {
write-output "Create Backup Encryption Key";

$MasterKeyEncPwd = (($Config).BackupEncryption).MasterKeyEncPwd;
$MasterKeyBackupPwd = (($Config).BackupEncryption).MasterKeyBackupPwd;
$CertName = (($Config).BackupEncryption).CertificateName -replace '\$Server', $Server;
$CertSubject = (($Config).BackupEncryption).CertificateSubject -replace '\$Server', $Server;
$CertExpiry = (($Config).BackupEncryption).CertificateExpiration;
$CertBackupEncPwd = (($Config).BackupEncryption).CertificateEncPwd;

$masterKey = "CREATE MASTER KEY ENCRYPTION BY PASSWORD = '$MasterKeyEncPwd';";
Invoke-sqlcmd -ServerInstance $Instance -Query $masterKey;

$serverCert = "CREATE CERTIFICATE [$CertName] WITH SUBJECT = '$CertSubject', EXPIRY_DATE = '$CertExpiry';";
Invoke-sqlcmd -ServerInstance $Instance -Query $serverCert;

$backupKey = "BACKUP SERVICE MASTER KEY TO FILE = '$BackupDrive`\Certificates\$Server\$Server`_service_master.key' ENCRYPTION BY PASSWORD = '$MasterKeyBackupPwd';"
Invoke-sqlcmd -ServerInstance $Instance -Query $backupKey;

$backupCert = "BACKUP CERTIFICATE [$CertName] TO FILE = '$BackupDrive`\Certificates\$Server\$Server`_DBBackupCert.cer' WITH PRIVATE KEY ( FILE = '$BackupDrive`\Certificates\$Server\$Server`_DBBackupCertKey.key', ENCRYPTION BY PASSWORD = '$CertBackupEncPwd');"
Invoke-sqlcmd -ServerInstance $Instance -Query $backupCert;
}
##############
# Harden SSL #
##############

Write-Output "Harden SSL on $Server (reboot required).";
Invoke-Command -ComputerName $Server -FilePath "$ScriptPath\Config_Scripts\iis8_ssl_hassharden_1.4.ps1";

} # End ForEach Server

#################
# Closing Notes #
#################
$continue = Read-Host "
Automated Setup Complete.
1. Add Backup Encryption Key Password to Password Vault.
2. Download & Install maintenance utilities (backup, reindex, checkdb) from http://minionware.net
Press Enter to Close";

